@@ -15,6 +15,7 @@
 use std::{collections::BTreeSet, fmt, sync::Arc};
 
 use as_variant::as_variant;
+use decryption_retry_task::DecryptionRetryTask;
 use eyeball_im::VectorDiff;
 use eyeball_im_util::vector::VectorObserverExt;
 use futures_core::Stream;
@@ -22,7 +23,7 @@ use imbl::Vector;
 #[cfg(test)]
 use matrix_sdk::{crypto::OlmMachine, SendOutsideWasm};
 use matrix_sdk::{
-    deserialized_responses::{TimelineEvent, TimelineEventKind as SdkTimelineEventKind},
+    deserialized_responses::TimelineEvent,
     event_cache::{
         paginator::{PaginationResult, Paginator},
         RoomEventCache,
@@ -48,18 +49,14 @@ use ruma::{
     TransactionId, UserId,
 };
 #[cfg(test)]
-use ruma::{events::receipt::ReceiptEventContent, RoomId};
+use ruma::{events::receipt::ReceiptEventContent, OwnedRoomId, RoomId};
 use tokio::sync::{RwLock, RwLockWriteGuard};
-use tracing::{
-    debug, error, field, field::debug, info, info_span, instrument, trace, warn, Instrument as _,
-};
+use tracing::{debug, error, field::debug, info, instrument, trace, warn};
 
-#[cfg(test)]
-pub(super) use self::observable_items::ObservableItems;
 pub(super) use self::{
     metadata::{RelativePosition, TimelineMetadata},
     observable_items::{
-        AllRemoteEvents, ObservableItemsEntry, ObservableItemsTransaction,
+        AllRemoteEvents, ObservableItems, ObservableItemsEntry, ObservableItemsTransaction,
         ObservableItemsTransactionEntry,
     },
     state::{FullEventMeta, PendingEdit, PendingEditKind, TimelineState},
@@ -70,10 +67,11 @@ use super::{
     event_handler::TimelineEventKind,
     event_item::{ReactionStatus, RemoteEventOrigin},
     item::TimelineUniqueId,
+    subscriber::TimelineSubscriber,
     traits::{Decryptor, RoomDataProvider},
-    DateDividerMode, Error, EventSendState, EventTimelineItem, InReplyToDetails, Message,
-    PaginationError, Profile, ReactionInfo, RepliedToEvent, TimelineDetails, TimelineEventItemId,
-    TimelineFocus, TimelineItem, TimelineItemContent, TimelineItemKind,
+    DateDividerMode, Error, EventSendState, EventTimelineItem, InReplyToDetails, PaginationError,
+    Profile, RepliedToEvent, TimelineDetails, TimelineEventItemId, TimelineFocus, TimelineItem,
+    TimelineItemContent, TimelineItemKind, VirtualTimelineItem,
 };
 use crate::{
     timeline::{
@@ -81,18 +79,21 @@ use crate::{
         date_dividers::DateDividerAdjuster,
         event_item::EventTimelineItemKind,
         pinned_events_loader::{PinnedEventsLoader, PinnedEventsLoaderError},
-        reactions::FullReactionKey,
-        TimelineEventFilterFn,
+        MsgLikeContent, MsgLikeKind, TimelineEventFilterFn,
     },
     unable_to_decrypt_hook::UtdHookManager,
 };
 
+mod aggregations;
+mod decryption_retry_task;
 mod metadata;
 mod observable_items;
 mod read_receipts;
 mod state;
 mod state_transaction;
 
+pub(super) use aggregations::*;
+
 /// Data associated to the current timeline focus.
 #[derive(Debug)]
 enum TimelineFocusData<P: RoomDataProvider> {
@@ -116,7 +117,7 @@ enum TimelineFocusData<P: RoomDataProvider> {
 }
 
 #[derive(Clone, Debug)]
-pub(super) struct TimelineController<P: RoomDataProvider = Room> {
+pub(super) struct TimelineController<P: RoomDataProvider = Room, D: Decryptor = Room> {
     /// Inner mutable state.
     state: Arc<RwLock<TimelineState>>,
 
@@ -130,6 +131,10 @@ pub(super) struct TimelineController<P: RoomDataProvider = Room> {
 
     /// Settings applied to this timeline.
     pub(super) settings: TimelineSettings,
+
+    /// Long-running task used to retry decryption of timeline items without
+    /// blocking main processing.
+    decryption_retry_task: DecryptionRetryTask<D>,
 }
 
 #[derive(Clone)]
@@ -170,7 +175,7 @@ impl Default for TimelineSettings {
 }
 
 #[derive(Debug, Clone, Copy)]
-enum TimelineFocusKind {
+pub(super) enum TimelineFocusKind {
     Live,
     Event,
     PinnedEvents,
@@ -255,13 +260,13 @@ pub fn default_event_filter(event: &AnySyncTimelineEvent, room_version: &RoomVer
     }
 }
 
-impl<P: RoomDataProvider> TimelineController<P> {
+impl<P: RoomDataProvider, D: Decryptor> TimelineController<P, D> {
     pub(super) fn new(
         room_data_provider: P,
         focus: TimelineFocus,
         internal_id_prefix: Option<String>,
         unable_to_decrypt_hook: Option<Arc<UtdHookManager>>,
-        is_room_encrypted: Option<bool>,
+        is_room_encrypted: bool,
     ) -> Self {
         let (focus_data, focus_kind) = match focus {
             TimelineFocus::Live => (TimelineFocusData::Live, TimelineFocusKind::Live),
@@ -286,20 +291,26 @@ impl<P: RoomDataProvider> TimelineController<P> {
             ),
         };
 
-        let state = TimelineState::new(
+        let state = Arc::new(RwLock::new(TimelineState::new(
             focus_kind,
             room_data_provider.own_user_id().to_owned(),
             room_data_provider.room_version(),
             internal_id_prefix,
             unable_to_decrypt_hook,
             is_room_encrypted,
-        );
+        )));
+
+        let settings = TimelineSettings::default();
+
+        let decryption_retry_task =
+            DecryptionRetryTask::new(state.clone(), room_data_provider.clone());
 
         Self {
-            state: Arc::new(RwLock::new(state)),
+            state,
             focus: Arc::new(RwLock::new(focus_data)),
             room_data_provider,
-            settings: Default::default(),
+            settings,
+            decryption_retry_task,
         }
     }
 
@@ -318,8 +329,7 @@ impl<P: RoomDataProvider> TimelineController<P> {
         match &*focus_guard {
             TimelineFocusData::Live => {
                 // Retrieve the cached events, and add them to the timeline.
-                let (events, _) =
-                    room_event_cache.subscribe().await.map_err(Error::EventCacheError)?;
+                let (events, _stream) = room_event_cache.subscribe().await;
 
                 let has_events = !events.is_empty();
 
@@ -377,23 +387,26 @@ impl<P: RoomDataProvider> TimelineController<P> {
     pub async fn handle_encryption_state_changes(&self) {
         let mut room_info = self.room_data_provider.room_info();
 
+        // Small function helper to help mark as encrypted.
+        let mark_encrypted = || async {
+            let mut state = self.state.write().await;
+            state.meta.is_room_encrypted = true;
+            state.mark_all_events_as_encrypted();
+        };
+
+        if room_info.get().encryption_state().is_encrypted() {
+            // If the room was already encrypted, it won't toggle to unencrypted, so we can
+            // shut down this task early.
+            mark_encrypted().await;
+            return;
+        }
+
         while let Some(info) = room_info.next().await {
-            let changed = {
-                let state = self.state.read().await;
-                let mut old_is_room_encrypted = state.meta.is_room_encrypted.write();
-                let is_encrypted_now = info.is_encrypted();
-
-                if *old_is_room_encrypted != Some(is_encrypted_now) {
-                    *old_is_room_encrypted = Some(is_encrypted_now);
-                    true
-                } else {
-                    false
-                }
-            };
-
-            if changed {
-                let mut state = self.state.write().await;
-                state.update_all_events_is_room_encrypted();
+            if info.encryption_state().is_encrypted() {
+                mark_encrypted().await;
+                // Once the room is encrypted, it cannot switch back to unencrypted, so our work
+                // here is done.
+                break;
             }
         }
     }
@@ -410,7 +423,28 @@ impl<P: RoomDataProvider> TimelineController<P> {
         }
     }
 
-    /// Run a backward pagination (in focused mode) and append the results to
+    /// Run a lazy backwards pagination (in live mode).
+    ///
+    /// It adjusts the `count` value of the `Skip` higher-order stream so that
+    /// more items are pushed front in the timeline.
+    ///
+    /// If no more items are available (i.e. if the `count` is zero), this
+    /// method returns `Some(needs)` where `needs` is the number of events that
+    /// must be unlazily backwards paginated.
+    pub(super) async fn live_lazy_paginate_backwards(&self, num_events: u16) -> Option<usize> {
+        let state = self.state.read().await;
+
+        let (count, needs) = state
+            .meta
+            .subscriber_skip_count
+            .compute_next_when_paginating_backwards(num_events.into());
+
+        state.meta.subscriber_skip_count.update(count, &state.timeline_focus);
+
+        needs
+    }
+
+    /// Run a backwards pagination (in focused mode) and append the results to
     /// the timeline.
     ///
     /// Returns whether we hit the start of the timeline.
@@ -439,7 +473,7 @@ impl<P: RoomDataProvider> TimelineController<P> {
         Ok(hit_end_of_timeline)
     }
 
-    /// Run a forward pagination (in focused mode) and append the results to
+    /// Run a forwards pagination (in focused mode) and append the results to
     /// the timeline.
     ///
     /// Returns whether we hit the end of the timeline.
@@ -486,21 +520,21 @@ impl<P: RoomDataProvider> TimelineController<P> {
     }
 
     #[cfg(test)]
-    pub(super) async fn subscribe(
+    pub(super) async fn subscribe_raw(
         &self,
     ) -> (
         Vector<Arc<TimelineItem>>,
         impl Stream<Item = VectorDiff<Arc<TimelineItem>>> + SendOutsideWasm,
     ) {
         let state = self.state.read().await;
-        (state.items.clone_items(), state.items.subscribe().into_stream())
+
+        state.items.subscribe().into_values_and_stream()
     }
 
-    pub(super) async fn subscribe_batched(
-        &self,
-    ) -> (Vector<Arc<TimelineItem>>, impl Stream<Item = Vec<VectorDiff<Arc<TimelineItem>>>>) {
+    pub(super) async fn subscribe(&self) -> (Vector<Arc<TimelineItem>>, TimelineSubscriber) {
         let state = self.state.read().await;
-        (state.items.clone_items(), state.items.subscribe().into_batched_stream())
+
+        TimelineSubscriber::new(&state.items, &state.meta.subscriber_skip_count)
     }
 
     pub(super) async fn subscribe_filter_map<U, F>(
@@ -532,6 +566,7 @@ impl<P: RoomDataProvider> TimelineController<P> {
 
         let user_id = self.room_data_provider.own_user_id();
         let prev_status = item
+            .content()
             .reactions()
             .get(key)
             .and_then(|group| group.get(user_id))
@@ -611,7 +646,7 @@ impl<P: RoomDataProvider> TimelineController<P> {
                     return Ok(false);
                 };
 
-                let mut reactions = item.reactions().clone();
+                let mut reactions = item.content().reactions().clone();
                 let reaction_info = reactions.remove_reaction(user_id, key);
 
                 if reaction_info.is_some() {
@@ -634,7 +669,7 @@ impl<P: RoomDataProvider> TimelineController<P> {
                             rfind_event_by_id(&state.items, &annotated_event_id)
                         {
                             // Re-add the reaction to the mapping.
-                            let mut reactions = item.reactions().clone();
+                            let mut reactions = item.content().reactions();
                             reactions
                                 .entry(key.to_owned())
                                 .or_default()
@@ -825,31 +860,49 @@ impl<P: RoomDataProvider> TimelineController<P> {
         });
 
         let Some((idx, item)) = result else {
-            // Event isn't found as a proper item. Try to find it as a reaction?
-            if let Some((event_id, reaction_key)) = new_event_id.zip(
-                txn.meta.reactions.map.get(&TimelineEventItemId::TransactionId(txn_id.to_owned())),
-            ) {
-                match &reaction_key.item {
-                    TimelineEventItemId::TransactionId(_) => {
-                        error!("unexpected remote reaction to local echo")
-                    }
-                    TimelineEventItemId::EventId(reacted_to_event_id) => {
-                        if let Some((item_pos, event_item)) =
-                            rfind_event_by_id(&txn.items, reacted_to_event_id)
-                        {
-                            let mut reactions = event_item.reactions().clone();
-                            if let Some(entry) = reactions
-                                .get_mut(&reaction_key.key)
-                                .and_then(|by_user| by_user.get_mut(&reaction_key.sender))
+            // Event wasn't found as a standalone item.
+            //
+            // If it was just sent, try to find if it matches a corresponding aggregation,
+            // and mark it as sent in that case.
+            if let Some(new_event_id) = new_event_id {
+                match txn
+                    .meta
+                    .aggregations
+                    .mark_aggregation_as_sent(txn_id.to_owned(), new_event_id.to_owned())
+                {
+                    MarkAggregationSentResult::MarkedSent { update } => {
+                        trace!("marked aggregation as sent");
+
+                        if let Some((target, aggregation)) = update {
+                            if let Some((item_pos, item)) =
+                                rfind_event_by_item_id(&txn.items, &target)
                             {
-                                trace!("updated reaction status to sent");
-                                entry.status = ReactionStatus::RemoteToRemote(event_id.to_owned());
-                                txn.items.replace(item_pos, event_item.with_reactions(reactions));
-                                txn.commit();
-                                return;
+                                let mut content = item.content().clone();
+                                match aggregation.apply(&mut content) {
+                                    ApplyAggregationResult::UpdatedItem => {
+                                        trace!("reapplied aggregation in the event");
+                                        let internal_id = item.internal_id.to_owned();
+                                        let new_item = item.with_content(content);
+                                        txn.items.replace(
+                                            item_pos,
+                                            TimelineItem::new(new_item, internal_id),
+                                        );
+                                        txn.commit();
+                                    }
+                                    ApplyAggregationResult::LeftItemIntact => {}
+                                    ApplyAggregationResult::Error(err) => {
+                                        warn!("when reapplying aggregation just marked as sent: {err}");
+                                    }
+                                }
                             }
                         }
+
+                        // Early return: we've found the event to mark as sent, it was an
+                        // aggregation.
+                        return;
                     }
+
+                    MarkAggregationSentResult::NotFound => {}
                 }
             }
 
@@ -858,7 +911,7 @@ impl<P: RoomDataProvider> TimelineController<P> {
         };
 
         let Some(local_item) = item.as_local() else {
-            warn!("We looked for a local item, but it transitioned to remote??");
+            warn!("We looked for a local item, but it transitioned to remote.");
             return;
         };
 
@@ -868,25 +921,10 @@ impl<P: RoomDataProvider> TimelineController<P> {
             error!(?existing_event_id, ?new_event_id, "Local echo already marked as sent");
         }
 
-        // If the event had local reactions, upgrade the mapping from reaction to
-        // events, to indicate that the event is now remote.
+        // If the event has just been marked as sent, update the aggregations mapping to
+        // take that into account.
         if let Some(new_event_id) = new_event_id {
-            let reactions = item.reactions();
-            for (_key, by_user) in reactions.iter() {
-                for (_user_id, info) in by_user.iter() {
-                    if let ReactionStatus::LocalToLocal(Some(reaction_handle)) = &info.status {
-                        let reaction_txn_id = reaction_handle.transaction_id().to_owned();
-                        if let Some(found) = txn
-                            .meta
-                            .reactions
-                            .map
-                            .get_mut(&TimelineEventItemId::TransactionId(reaction_txn_id))
-                        {
-                            found.item = TimelineEventItemId::EventId(new_event_id.to_owned());
-                        }
-                    }
-                }
-            }
+            txn.meta.aggregations.mark_target_as_sent(txn_id.to_owned(), new_event_id.to_owned());
         }
 
         let new_item = item.with_inner_kind(local_item.with_send_state(send_state));
@@ -914,41 +952,42 @@ impl<P: RoomDataProvider> TimelineController<P> {
 
             txn.commit();
 
-            debug!("Discarded local echo");
+            debug!("discarded local echo");
             return true;
         }
 
-        // Look if this was a local reaction echo.
-        if let Some(full_key) =
-            state.meta.reactions.map.remove(&TimelineEventItemId::TransactionId(txn_id.to_owned()))
-        {
-            let item = match &full_key.item {
-                TimelineEventItemId::TransactionId(txn_id) => {
-                    rfind_event_item(&state.items, |item| item.transaction_id() == Some(txn_id))
-                }
-                TimelineEventItemId::EventId(event_id) => rfind_event_by_id(&state.items, event_id),
-            };
+        // Avoid multiple mutable and immutable borrows of the lock guard by explicitly
+        // dereferencing it once.
+        let state = &mut *state;
 
-            let Some((idx, item)) = item else {
-                warn!("missing reacted-to item for a reaction");
+        // Look if this was a local aggregation.
+        if let Some((target, aggregation)) = state
+            .meta
+            .aggregations
+            .try_remove_aggregation(&TimelineEventItemId::TransactionId(txn_id.to_owned()))
+        {
+            let Some((item_pos, item)) = rfind_event_by_item_id(&state.items, target) else {
+                warn!("missing target item for a local aggregation");
                 return false;
             };
 
-            let mut reactions = item.reactions().clone();
-            if reactions.remove_reaction(&full_key.sender, &full_key.key).is_some() {
-                let updated_item = item.with_reactions(reactions);
-                state.items.replace(idx, updated_item);
-            } else {
-                warn!(
-                    "missing reaction {} for sender {} on timeline item",
-                    full_key.key, full_key.sender
-                );
+            let mut content = item.content().clone();
+            match aggregation.unapply(&mut content) {
+                ApplyAggregationResult::UpdatedItem => {
+                    trace!("removed local reaction to local echo");
+                    let internal_id = item.internal_id.clone();
+                    let new_item = item.with_content(content);
+                    state.items.replace(item_pos, TimelineItem::new(new_item, internal_id));
+                }
+                ApplyAggregationResult::LeftItemIntact => {}
+                ApplyAggregationResult::Error(err) => {
+                    warn!("when undoing local aggregation: {err}");
+                }
             }
 
             return true;
         }
 
-        debug!("Can't find local echo to discard");
         false
     }
 
@@ -987,10 +1026,17 @@ impl<P: RoomDataProvider> TimelineController<P> {
         };
 
         // Replace the local-related state (kind) and the content state.
+        let prev_reactions = prev_item.content().reactions();
+        let prev_thread_root = prev_item.content().thread_root();
+        let prev_in_reply_to = prev_item.content().in_reply_to();
         let new_item = TimelineItem::new(
-            prev_item
-                .with_kind(ti_kind)
-                .with_content(TimelineItemContent::message(content, None, &txn.items)),
+            prev_item.with_kind(ti_kind).with_content(TimelineItemContent::message(
+                content,
+                None,
+                prev_reactions,
+                prev_thread_root,
+                prev_in_reply_to,
+            )),
             prev_item.internal_id.to_owned(),
         );
 
@@ -1005,144 +1051,12 @@ impl<P: RoomDataProvider> TimelineController<P> {
         true
     }
 
-    #[instrument(skip(self, room), fields(room_id = ?room.room_id()))]
-    pub(super) async fn retry_event_decryption(
-        &self,
-        room: &Room,
-        session_ids: Option<BTreeSet<String>>,
-    ) {
-        self.retry_event_decryption_inner(room.to_owned(), session_ids).await
-    }
-
-    #[cfg(test)]
-    pub(super) async fn retry_event_decryption_test(
-        &self,
-        room_id: &RoomId,
-        olm_machine: OlmMachine,
-        session_ids: Option<BTreeSet<String>>,
-    ) {
-        self.retry_event_decryption_inner((olm_machine, room_id.to_owned()), session_ids).await
-    }
-
     async fn retry_event_decryption_inner(
         &self,
-        decryptor: impl Decryptor,
+        decryptor: D,
         session_ids: Option<BTreeSet<String>>,
     ) {
-        use super::EncryptedMessage;
-
-        let mut state = self.state.clone().write_owned().await;
-
-        let should_retry = move |session_id: &str| {
-            if let Some(session_ids) = &session_ids {
-                session_ids.contains(session_id)
-            } else {
-                true
-            }
-        };
-
-        let retry_indices: Vec<_> = state
-            .items
-            .iter()
-            .enumerate()
-            .filter_map(|(idx, item)| match item.as_event()?.content().as_unable_to_decrypt()? {
-                EncryptedMessage::MegolmV1AesSha2 { session_id, .. }
-                    if should_retry(session_id) =>
-                {
-                    Some(idx)
-                }
-                EncryptedMessage::MegolmV1AesSha2 { .. }
-                | EncryptedMessage::OlmV1Curve25519AesSha2 { .. }
-                | EncryptedMessage::Unknown => None,
-            })
-            .collect();
-
-        if retry_indices.is_empty() {
-            return;
-        }
-
-        debug!("Retrying decryption");
-
-        let settings = self.settings.clone();
-        let room_data_provider = self.room_data_provider.clone();
-        let push_rules_context = room_data_provider.push_rules_and_context().await;
-        let unable_to_decrypt_hook = state.meta.unable_to_decrypt_hook.clone();
-
-        matrix_sdk::executor::spawn(async move {
-            let retry_one = |item: Arc<TimelineItem>| {
-                let decryptor = decryptor.clone();
-                let should_retry = &should_retry;
-                let unable_to_decrypt_hook = unable_to_decrypt_hook.clone();
-                async move {
-                    let event_item = item.as_event()?;
-
-                    let session_id = match event_item.content().as_unable_to_decrypt()? {
-                        EncryptedMessage::MegolmV1AesSha2 { session_id, .. }
-                            if should_retry(session_id) =>
-                        {
-                            session_id
-                        }
-                        EncryptedMessage::MegolmV1AesSha2 { .. }
-                        | EncryptedMessage::OlmV1Curve25519AesSha2 { .. }
-                        | EncryptedMessage::Unknown => return None,
-                    };
-
-                    tracing::Span::current().record("session_id", session_id);
-
-                    let Some(remote_event) = event_item.as_remote() else {
-                        error!("Key for unable-to-decrypt timeline item is not an event ID");
-                        return None;
-                    };
-
-                    tracing::Span::current().record("event_id", debug(&remote_event.event_id));
-
-                    let Some(original_json) = &remote_event.original_json else {
-                        error!("UTD item must contain original JSON");
-                        return None;
-                    };
-
-                    match decryptor.decrypt_event_impl(original_json).await {
-                        Ok(event) => {
-                            if let SdkTimelineEventKind::UnableToDecrypt { utd_info, .. } =
-                                event.kind
-                            {
-                                info!(
-                                    "Failed to decrypt event after receiving room key: {:?}",
-                                    utd_info.reason
-                                );
-                                None
-                            } else {
-                                // Notify observers that we managed to eventually decrypt an event.
-                                if let Some(hook) = unable_to_decrypt_hook {
-                                    hook.on_late_decrypt(&remote_event.event_id).await;
-                                }
-
-                                Some(event)
-                            }
-                        }
-                        Err(e) => {
-                            info!("Failed to decrypt event after receiving room key: {e}");
-                            None
-                        }
-                    }
-                }
-                .instrument(info_span!(
-                    "retry_one",
-                    session_id = field::Empty,
-                    event_id = field::Empty
-                ))
-            };
-
-            state
-                .retry_event_decryption(
-                    retry_one,
-                    retry_indices,
-                    push_rules_context,
-                    &room_data_provider,
-                    &settings,
-                )
-                .await;
-        });
+        self.decryption_retry_task.decrypt(decryptor, session_ids, self.settings.clone()).await;
     }
 
     pub(super) async fn set_sender_profiles_pending(&self) {
@@ -1303,7 +1217,9 @@ impl<P: RoomDataProvider> TimelineController<P> {
                     self.update_event_send_state(
                         &echo.transaction_id,
                         EventSendState::SendingFailed {
-                            error: Arc::new(matrix_sdk::Error::SendQueueWedgeError(send_error)),
+                            error: Arc::new(matrix_sdk::Error::SendQueueWedgeError(Box::new(
+                                send_error,
+                            ))),
                             is_recoverable: false,
                         },
                     )
@@ -1326,39 +1242,26 @@ impl<P: RoomDataProvider> TimelineController<P> {
         applies_to: OwnedTransactionId,
     ) {
         let mut state = self.state.write().await;
+        let mut tr = state.transaction();
 
-        let Some((item_pos, item)) =
-            rfind_event_item(&state.items, |item| item.transaction_id() == Some(&applies_to))
-        else {
-            warn!("Local item not found anymore.");
-            return;
-        };
-
-        let user_id = self.room_data_provider.own_user_id();
+        let target = TimelineEventItemId::TransactionId(applies_to);
 
         let reaction_txn_id = send_handle.transaction_id().to_owned();
-        let reaction_info = ReactionInfo {
-            timestamp: MilliSecondsSinceUnixEpoch::now(),
-            status: ReactionStatus::LocalToLocal(Some(send_handle)),
-        };
-
-        let mut reactions = item.reactions().clone();
-        let by_user = reactions.entry(reaction_key.clone()).or_default();
-        by_user.insert(user_id.to_owned(), reaction_info);
-
-        trace!("Adding local reaction to local echo");
-        let new_item = item.with_reactions(reactions);
-        state.items.replace(item_pos, new_item);
-
-        // Add it to the reaction map, so we can discard it later if needs be.
-        state.meta.reactions.map.insert(
+        let reaction_status = ReactionStatus::LocalToLocal(Some(send_handle));
+        let aggregation = Aggregation::new(
             TimelineEventItemId::TransactionId(reaction_txn_id),
-            FullReactionKey {
-                item: TimelineEventItemId::TransactionId(applies_to),
-                key: reaction_key,
-                sender: user_id.to_owned(),
+            AggregationKind::Reaction {
+                key: reaction_key.clone(),
+                sender: self.room_data_provider.own_user_id().to_owned(),
+                timestamp: MilliSecondsSinceUnixEpoch::now(),
+                reaction_status,
             },
         );
+
+        tr.meta.aggregations.add(target.clone(), aggregation.clone());
+        find_item_and_apply_aggregation(&mut tr.items, &target, aggregation);
+
+        tr.commit();
     }
 
     /// Handle a single room send queue update.
@@ -1411,6 +1314,18 @@ impl<P: RoomDataProvider> TimelineController<P> {
             }
         }
     }
+
+    /// Insert a timeline start item at the beginning of the room, if it's
+    /// missing.
+    pub async fn insert_timeline_start_if_missing(&self) {
+        let mut state = self.state.write().await;
+        let mut txn = state.transaction();
+        if txn.items.get(0).is_some_and(|item| item.is_timeline_start()) {
+            return;
+        }
+        txn.items.push_front(txn.meta.new_timeline_item(VirtualTimelineItem::TimelineStart), None);
+        txn.commit();
+    }
 }
 
 impl TimelineController {
@@ -1430,11 +1345,11 @@ impl TimelineController {
             .ok_or(Error::EventNotInTimeline(TimelineEventItemId::EventId(event_id.to_owned())))?
             .clone();
 
-        let TimelineItemContent::Message(message) = item.content().clone() else {
+        let TimelineItemContent::MsgLike(msglike) = item.content().clone() else {
             debug!("Event is not a message");
             return Ok(());
         };
-        let Some(in_reply_to) = message.in_reply_to() else {
+        let Some(in_reply_to) = msglike.in_reply_to.clone() else {
             debug!("Event is not a reply");
             return Ok(());
         };
@@ -1454,7 +1369,7 @@ impl TimelineController {
             index,
             &item,
             internal_id,
-            &message,
+            &msglike,
             &in_reply_to.event_id,
             self.room(),
         )
@@ -1468,11 +1383,17 @@ impl TimelineController {
 
         // Check the state of the event again, it might have been redacted while
         // the request was in-flight.
-        let TimelineItemContent::Message(message) = item.content().clone() else {
+        let TimelineItemContent::MsgLike(MsgLikeContent {
+            kind: MsgLikeKind::Message(message),
+            reactions,
+            thread_root,
+            in_reply_to,
+        }) = item.content().clone()
+        else {
             info!("Event is no longer a message (redacted?)");
             return Ok(());
         };
-        let Some(in_reply_to) = message.in_reply_to() else {
+        let Some(in_reply_to) = in_reply_to else {
             warn!("Event no longer has a reply (bug?)");
             return Ok(());
         };
@@ -1482,12 +1403,12 @@ impl TimelineController {
         trace!("Updating in-reply-to details");
         let internal_id = item.internal_id.to_owned();
         let mut item = item.clone();
-        item.set_content(TimelineItemContent::Message(
-            message.with_in_reply_to(InReplyToDetails {
-                event_id: in_reply_to.event_id.clone(),
-                event,
-            }),
-        ));
+        item.set_content(TimelineItemContent::MsgLike(MsgLikeContent {
+            kind: MsgLikeKind::Message(message),
+            reactions,
+            thread_root,
+            in_reply_to: Some(InReplyToDetails { event_id: in_reply_to.event_id, event }),
+        }));
         state.items.replace(index, TimelineItem::new(item, internal_id));
 
         Ok(())
@@ -1576,6 +1497,23 @@ impl TimelineController {
         let state = self.state.read().await;
         state.items.all_remote_events().last().map(|event_meta| &event_meta.event_id).cloned()
     }
+
+    #[instrument(skip(self), fields(room_id = ?self.room().room_id()))]
+    pub(super) async fn retry_event_decryption(&self, session_ids: Option<BTreeSet<String>>) {
+        self.retry_event_decryption_inner(self.room().to_owned(), session_ids).await
+    }
+}
+
+#[cfg(test)]
+impl<P: RoomDataProvider> TimelineController<P, (OlmMachine, OwnedRoomId)> {
+    pub(super) async fn retry_event_decryption_test(
+        &self,
+        room_id: &RoomId,
+        olm_machine: OlmMachine,
+        session_ids: Option<BTreeSet<String>>,
+    ) {
+        self.retry_event_decryption_inner((olm_machine, room_id.to_owned()), session_ids).await
+    }
 }
 
 async fn fetch_replied_to_event(
@@ -1583,17 +1521,12 @@ async fn fetch_replied_to_event(
     index: usize,
     item: &EventTimelineItem,
     internal_id: TimelineUniqueId,
-    message: &Message,
+    msglike: &MsgLikeContent,
     in_reply_to: &EventId,
     room: &Room,
 ) -> Result<TimelineDetails<Box<RepliedToEvent>>, Error> {
     if let Some((_, item)) = rfind_event_by_id(&state.items, in_reply_to) {
-        let details = TimelineDetails::Ready(Box::new(RepliedToEvent {
-            content: item.content.clone(),
-            sender: item.sender().to_owned(),
-            sender_profile: item.sender_profile().clone(),
-        }));
-
+        let details = TimelineDetails::Ready(Box::new(RepliedToEvent::from_timeline_item(&item)));
         trace!("Found replied-to event locally");
         return Ok(details);
     };
@@ -1601,11 +1534,11 @@ async fn fetch_replied_to_event(
     // Replace the item with a new timeline item that has the fetching status of the
     // replied-to event to pending.
     trace!("Setting in-reply-to details to pending");
-    let reply = message.with_in_reply_to(InReplyToDetails {
-        event_id: in_reply_to.to_owned(),
-        event: TimelineDetails::Pending,
-    });
-    let event_item = item.with_content(TimelineItemContent::Message(reply));
+    let in_reply_to_details =
+        InReplyToDetails { event_id: in_reply_to.to_owned(), event: TimelineDetails::Pending };
+
+    let event_item = item
+        .with_content(TimelineItemContent::MsgLike(msglike.with_in_reply_to(in_reply_to_details)));
 
     let new_timeline_item = TimelineItem::new(event_item, internal_id);
     state.items.replace(index, new_timeline_item);
@@ -1614,7 +1547,7 @@ async fn fetch_replied_to_event(
     drop(state);
 
     trace!("Fetching replied-to event");
-    let res = match room.event(in_reply_to, None).await {
+    let res = match room.load_or_fetch_event(in_reply_to, None).await {
         Ok(timeline_event) => TimelineDetails::Ready(Box::new(
             RepliedToEvent::try_from_timeline_event(timeline_event, room).await?,
         )),
