@@ -38,7 +38,7 @@ use ruma::{
         room::{
             encrypted::RoomEncryptedEventContent,
             member::RoomMemberEventContent,
-            message::{self, RoomMessageEventContent, RoomMessageEventContentWithoutRelation},
+            message::{Relation, RoomMessageEventContent, RoomMessageEventContentWithoutRelation},
         },
         AnyMessageLikeEventContent, AnySyncMessageLikeEvent, AnySyncStateEvent,
         AnySyncTimelineEvent, BundledMessageLikeRelations, EventContent, FullStateEventContent,
@@ -51,22 +51,22 @@ use ruma::{
 use tracing::{debug, error, field::debug, info, instrument, trace, warn};
 
 use super::{
-    algorithms::rfind_event_by_id,
+    algorithms::{rfind_event_by_id, rfind_event_by_item_id},
     controller::{
-        ObservableItemsTransaction, ObservableItemsTransactionEntry, PendingEdit, PendingEditKind,
-        TimelineMetadata, TimelineStateTransaction,
+        find_item_and_apply_aggregation, Aggregation, AggregationKind, ApplyAggregationResult,
+        ObservableItemsTransaction, PendingEdit, PendingEditKind, TimelineMetadata,
+        TimelineStateTransaction,
     },
     date_dividers::DateDividerAdjuster,
     event_item::{
         extract_bundled_edit_event_json, extract_poll_edit_content, extract_room_msg_edit_content,
         AnyOtherFullStateEventContent, EventSendState, EventTimelineItemKind,
-        LocalEventTimelineItem, PollState, Profile, ReactionInfo, ReactionStatus,
-        ReactionsByKeyBySender, RemoteEventOrigin, RemoteEventTimelineItem, TimelineEventItemId,
+        LocalEventTimelineItem, PollState, Profile, RemoteEventOrigin, RemoteEventTimelineItem,
+        TimelineEventItemId,
     },
-    reactions::{FullReactionKey, PendingReaction},
     traits::RoomDataProvider,
-    EventTimelineItem, InReplyToDetails, OtherState, RepliedToEvent, Sticker, TimelineDetails,
-    TimelineItem, TimelineItemContent,
+    EncryptedMessage, EventTimelineItem, InReplyToDetails, MsgLikeContent, MsgLikeKind, OtherState,
+    ReactionStatus, RepliedToEvent, Sticker, TimelineDetails, TimelineItem, TimelineItemContent,
 };
 use crate::events::SyncTimelineEventWithoutContent;
 
@@ -104,6 +104,14 @@ impl Flow {
         as_variant!(self, Flow::Remote { event_id, .. } => event_id)
     }
 
+    /// Returns the [`TimelineEventItemId`] associated to this future item.
+    pub(crate) fn timeline_item_id(&self) -> TimelineEventItemId {
+        match self {
+            Flow::Remote { event_id, .. } => TimelineEventItemId::EventId(event_id.clone()),
+            Flow::Local { txn_id, .. } => TimelineEventItemId::TransactionId(txn_id.clone()),
+        }
+    }
+
     /// If the flow is remote, returns the associated full raw event.
     pub(crate) fn raw_event(&self) -> Option<&Raw<AnySyncTimelineEvent>> {
         as_variant!(self, Flow::Remote { raw_event, .. } => raw_event)
@@ -390,7 +398,7 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
                 }
 
                 AnyMessageLikeEventContent::RoomMessage(RoomMessageEventContent {
-                    relates_to: Some(message::Relation::Replacement(re)),
+                    relates_to: Some(Relation::Replacement(re)),
                     ..
                 }) => {
                     self.handle_room_message_edit(re);
@@ -404,7 +412,15 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
 
                 AnyMessageLikeEventContent::Sticker(content) => {
                     if should_add {
-                        self.add_item(TimelineItemContent::Sticker(Sticker { content }), None);
+                        self.add_item(
+                            TimelineItemContent::MsgLike(MsgLikeContent {
+                                kind: MsgLikeKind::Sticker(Sticker { content }),
+                                reactions: Default::default(),
+                                thread_root: None,
+                                in_reply_to: None,
+                            }),
+                            None,
+                        );
                     }
                 }
 
@@ -448,7 +464,12 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
             TimelineEventKind::UnableToDecrypt { content, utd_cause } => {
                 // TODO: Handle replacements if the replaced event is also UTD
                 if should_add {
-                    self.add_item(TimelineItemContent::unable_to_decrypt(content, utd_cause), None);
+                    self.add_item(
+                        TimelineItemContent::MsgLike(MsgLikeContent::unable_to_decrypt(
+                            EncryptedMessage::from_content(content, utd_cause),
+                        )),
+                        None,
+                    );
                 }
 
                 // Let the hook know that we ran into an unable-to-decrypt that is added to the
@@ -463,7 +484,7 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
 
             TimelineEventKind::RedactedMessage { event_type } => {
                 if event_type != MessageLikeEventType::Reaction && should_add {
-                    self.add_item(TimelineItemContent::RedactedMessage, None);
+                    self.add_item(TimelineItemContent::MsgLike(MsgLikeContent::redacted()), None);
                 }
             }
 
@@ -562,9 +583,48 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
             .or(pending_edit)
             .unzip();
 
+        let mut replied_to_event_id = None;
+        let mut thread_root = None;
+        let in_reply_to_details = msg.relates_to.as_ref().and_then(|relation| match relation {
+            Relation::Reply { in_reply_to } => {
+                replied_to_event_id = Some(in_reply_to.event_id.clone());
+                Some(InReplyToDetails::new(in_reply_to.event_id.clone(), self.items))
+            }
+            Relation::Thread(thread) => {
+                thread_root = Some(thread.event_id.clone());
+                thread.in_reply_to.as_ref().map(|in_reply_to| {
+                    replied_to_event_id = Some(in_reply_to.event_id.clone());
+                    InReplyToDetails::new(in_reply_to.event_id.clone(), self.items)
+                })
+            }
+            _ => None,
+        });
+
+        // If this message is a reply to another message, add an entry in the inverted
+        // mapping.
+        if let Some(event_id) = self.ctx.flow.event_id() {
+            if let Some(replied_to_event_id) = replied_to_event_id {
+                // This is a reply! Add an entry.
+                self.meta
+                    .replies
+                    .entry(replied_to_event_id)
+                    .or_default()
+                    .insert(event_id.to_owned());
+            }
+        }
+
         let edit_json = edit_json.flatten();
 
-        self.add_item(TimelineItemContent::message(msg, edit_content, self.items), edit_json);
+        self.add_item(
+            TimelineItemContent::message(
+                msg,
+                edit_content,
+                Default::default(),
+                thread_root,
+                in_reply_to_details,
+            ),
+            edit_json,
+        );
     }
 
     #[instrument(skip_all, fields(replacement_event_id = ?replacement.event_id))]
@@ -580,7 +640,12 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
                 let internal_id = item.internal_id.to_owned();
 
                 // Update all events that replied to this message with the edited content.
-                Self::maybe_update_responses(self.items, &replacement.event_id, &new_item);
+                Self::maybe_update_responses(
+                    self.meta,
+                    self.items,
+                    &replacement.event_id,
+                    &new_item,
+                );
 
                 // Update the event itself.
                 self.items.replace(item_pos, TimelineItem::new(new_item, internal_id));
@@ -674,7 +739,13 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
             return None;
         }
 
-        let TimelineItemContent::Message(msg) = item.content() else {
+        let TimelineItemContent::MsgLike(MsgLikeContent {
+            kind: MsgLikeKind::Message(msg),
+            reactions,
+            thread_root,
+            in_reply_to,
+        }) = item.content()
+        else {
             info!(
                 "Edit of message event applies to {:?}, discarding",
                 item.content().debug_string(),
@@ -685,8 +756,15 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
         let mut new_msg = msg.clone();
         new_msg.apply_edit(new_content);
 
-        let mut new_item =
-            item.with_content_and_latest_edit(TimelineItemContent::Message(new_msg), edit_json);
+        let mut new_item = item.with_content_and_latest_edit(
+            TimelineItemContent::MsgLike(MsgLikeContent {
+                kind: MsgLikeKind::Message(new_msg),
+                reactions: reactions.clone(),
+                thread_root: thread_root.clone(),
+                in_reply_to: in_reply_to.clone(),
+            }),
+            edit_json,
+        );
 
         if let Flow::Remote { encryption_info, .. } = &self.ctx.flow {
             new_item = new_item.with_encryption_info(encryption_info.clone());
@@ -695,83 +773,39 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
         Some(new_item)
     }
 
-    // Redacted reaction events are no-ops so don't need to be handled
+    /// Apply a reaction to a *remote* event.
+    ///
+    /// Reactions to local events are applied in
+    /// [`crate::timeline::TimelineController::handle_local_echo`].
     #[instrument(skip_all, fields(relates_to_event_id = ?c.relates_to.event_id))]
     fn handle_reaction(&mut self, c: ReactionEventContent) {
-        let reacted_to_event_id = &c.relates_to.event_id;
+        let target = TimelineEventItemId::EventId(c.relates_to.event_id);
 
-        let (reaction_id, send_handle, old_txn_id) = match &self.ctx.flow {
-            Flow::Local { txn_id, send_handle, .. } => {
-                (TimelineEventItemId::TransactionId(txn_id.clone()), send_handle.clone(), None)
+        // Add the aggregation to the manager.
+        let reaction_status = match &self.ctx.flow {
+            Flow::Local { send_handle, .. } => {
+                // This is a local echo for a reaction to a remote event.
+                ReactionStatus::LocalToRemote(send_handle.clone())
             }
-            Flow::Remote { event_id, txn_id, .. } => {
-                (TimelineEventItemId::EventId(event_id.clone()), None, txn_id.as_ref())
+            Flow::Remote { event_id, .. } => {
+                // This is the remote echo for a reaction to a remote event.
+                ReactionStatus::RemoteToRemote(event_id.clone())
             }
         };
-
-        if let Some((idx, event_item)) = rfind_event_by_id(self.items, reacted_to_event_id) {
-            // Ignore reactions on redacted events.
-            if let TimelineItemContent::RedactedMessage = event_item.content() {
-                debug!("Ignoring reaction on redacted event");
-                return;
-            }
-
-            trace!("Added reaction");
-
-            // Add the reaction to the event item's bundled reactions.
-            let mut reactions = event_item.reactions.clone();
-
-            reactions.entry(c.relates_to.key.clone()).or_default().insert(
-                self.ctx.sender.clone(),
-                ReactionInfo {
-                    timestamp: self.ctx.timestamp,
-                    status: match &reaction_id {
-                        TimelineEventItemId::TransactionId(_txn_id) => {
-                            ReactionStatus::LocalToRemote(send_handle)
-                        }
-                        TimelineEventItemId::EventId(event_id) => {
-                            ReactionStatus::RemoteToRemote(event_id.clone())
-                        }
-                    },
-                },
-            );
-
-            self.items.replace(idx, event_item.with_reactions(reactions));
-
-            self.result.items_updated += 1;
-        } else {
-            trace!("Timeline item not found, adding reaction to the pending list");
-
-            let TimelineEventItemId::EventId(reaction_event_id) = reaction_id.clone() else {
-                error!("Adding local reaction echo to event absent from the timeline");
-                return;
-            };
-
-            self.meta.reactions.pending.entry(reacted_to_event_id.to_owned()).or_default().insert(
-                reaction_event_id,
-                PendingReaction {
-                    key: c.relates_to.key.clone(),
-                    sender_id: self.ctx.sender.clone(),
-                    timestamp: self.ctx.timestamp,
-                },
-            );
-        }
-
-        if let Some(txn_id) = old_txn_id {
-            // Try to remove a local echo of that reaction. It might be missing if the
-            // reaction wasn't sent by this device, or was sent in a previous
-            // session.
-            self.meta.reactions.map.remove(&TimelineEventItemId::TransactionId(txn_id.clone()));
-        }
-
-        self.meta.reactions.map.insert(
-            reaction_id,
-            FullReactionKey {
-                item: TimelineEventItemId::EventId(c.relates_to.event_id),
-                sender: self.ctx.sender.clone(),
+        let aggregation = Aggregation::new(
+            self.ctx.flow.timeline_item_id(),
+            AggregationKind::Reaction {
                 key: c.relates_to.key,
+                sender: self.ctx.sender.clone(),
+                timestamp: self.ctx.timestamp,
+                reaction_status,
             },
         );
+
+        self.meta.aggregations.add(target.clone(), aggregation.clone());
+        if find_item_and_apply_aggregation(self.items, &target, aggregation) {
+            self.result.items_updated += 1;
+        }
     }
 
     #[instrument(skip_all, fields(replacement_event_id = ?replacement.event_id))]
@@ -818,13 +852,24 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
             return None;
         }
 
-        let TimelineItemContent::Poll(poll_state) = &item.content() else {
+        let TimelineItemContent::MsgLike(MsgLikeContent {
+            kind: MsgLikeKind::Poll(poll_state),
+            reactions,
+            thread_root,
+            in_reply_to,
+        }) = &item.content()
+        else {
             info!("Edit of poll event applies to {}, discarding", item.content().debug_string(),);
             return None;
         };
 
         let new_content = match poll_state.edit(replacement.new_content) {
-            Some(edited_poll_state) => TimelineItemContent::Poll(edited_poll_state),
+            Some(edited_poll_state) => TimelineItemContent::MsgLike(MsgLikeContent {
+                kind: MsgLikeKind::Poll(edited_poll_state),
+                reactions: reactions.clone(),
+                thread_root: thread_root.clone(),
+                in_reply_to: in_reply_to.clone(),
+            }),
             None => {
                 info!("Not applying edit to a poll that's already ended");
                 return None;
@@ -866,76 +911,52 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
             .or(pending_edit)
             .unzip();
 
-        let mut poll_state = PollState::new(c, edit_content);
-
-        if let Some(event_id) = self.ctx.flow.event_id() {
-            // Applying the cache to remote events only because local echoes
-            // don't have an event ID that could be referenced by responses yet.
-            self.meta.pending_poll_events.apply_pending(event_id, &mut poll_state);
-        }
+        let poll_state = PollState::new(c, edit_content);
 
         let edit_json = edit_json.flatten();
 
-        self.add_item(TimelineItemContent::Poll(poll_state), edit_json);
+        self.add_item(
+            TimelineItemContent::MsgLike(MsgLikeContent {
+                kind: MsgLikeKind::Poll(poll_state),
+                reactions: Default::default(),
+                thread_root: None,
+                in_reply_to: None,
+            }),
+            edit_json,
+        );
     }
 
     fn handle_poll_response(&mut self, c: UnstablePollResponseEventContent) {
-        let Some((item_pos, item)) = rfind_event_by_id(self.items, &c.relates_to.event_id) else {
-            self.meta.pending_poll_events.add_response(
-                &c.relates_to.event_id,
-                &self.ctx.sender,
-                self.ctx.timestamp,
-                &c,
-            );
-            return;
-        };
-
-        let TimelineItemContent::Poll(poll_state) = item.content() else {
-            return;
-        };
-
-        let new_item = item.with_content(TimelineItemContent::Poll(poll_state.add_response(
-            &self.ctx.sender,
-            self.ctx.timestamp,
-            &c,
-        )));
-
-        trace!("Adding poll response.");
-        self.items.replace(item_pos, TimelineItem::new(new_item, item.internal_id.to_owned()));
-        self.result.items_updated += 1;
+        let target = TimelineEventItemId::EventId(c.relates_to.event_id);
+        let aggregation = Aggregation::new(
+            self.ctx.flow.timeline_item_id(),
+            AggregationKind::PollResponse {
+                sender: self.ctx.sender.clone(),
+                timestamp: self.ctx.timestamp,
+                answers: c.poll_response.answers,
+            },
+        );
+        self.meta.aggregations.add(target.clone(), aggregation.clone());
+        if find_item_and_apply_aggregation(self.items, &target, aggregation) {
+            self.result.items_updated += 1;
+        }
     }
 
     fn handle_poll_end(&mut self, c: UnstablePollEndEventContent) {
-        let Some((item_pos, item)) = rfind_event_by_id(self.items, &c.relates_to.event_id) else {
-            self.meta.pending_poll_events.mark_as_ended(&c.relates_to.event_id, self.ctx.timestamp);
-            return;
-        };
-
-        let TimelineItemContent::Poll(poll_state) = item.content() else {
-            return;
-        };
-
-        match poll_state.end(self.ctx.timestamp) {
-            Ok(poll_state) => {
-                let new_item = item.with_content(TimelineItemContent::Poll(poll_state));
-
-                trace!("Ending poll.");
-                self.items
-                    .replace(item_pos, TimelineItem::new(new_item, item.internal_id.to_owned()));
-                self.result.items_updated += 1;
-            }
-            Err(_) => {
-                info!("Got multiple poll end events, discarding");
-            }
+        let target = TimelineEventItemId::EventId(c.relates_to.event_id);
+        let aggregation = Aggregation::new(
+            self.ctx.flow.timeline_item_id(),
+            AggregationKind::PollEnd { end_date: self.ctx.timestamp },
+        );
+        self.meta.aggregations.add(target.clone(), aggregation.clone());
+        if find_item_and_apply_aggregation(self.items, &target, aggregation) {
+            self.result.items_updated += 1;
         }
     }
 
     /// Looks for the redacted event in all the timeline event items, and
     /// redacts it.
     ///
-    /// This only applies to *remote* events; for local items being redacted,
-    /// use [`Self::handle_reaction_redaction`].
-    ///
     /// This assumes the redacted event was present in the timeline in the first
     /// place; it will warn if the redacted event has not been found.
     #[instrument(skip_all, fields(redacts_event_id = ?redacted))]
@@ -943,8 +964,8 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
         // TODO: Apply local redaction of PollResponse and PollEnd events.
         // https://github.com/matrix-org/matrix-rust-sdk/pull/2381#issuecomment-1689647825
 
-        // If it's a reaction that's being redacted, handle it here.
-        if self.handle_reaction_redaction(TimelineEventItemId::EventId(redacted.clone())) {
+        // If it's an aggregation that's being redacted, handle it here.
+        if self.handle_aggregation_redaction(redacted.clone()) {
             // When we have raw timeline items, we should not return here anymore, as we
             // might need to redact the raw item as well.
             return;
@@ -953,7 +974,7 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
         // General path: redact another kind of (non-reaction) event.
         if let Some((idx, item)) = rfind_event_by_id(self.items, &redacted) {
             if item.as_remote().is_some() {
-                if let TimelineItemContent::RedactedMessage = &item.content {
+                if item.content.is_redacted() {
                     debug!("event item is already redacted");
                 } else {
                     let new_item = item.redact(&self.meta.room_version);
@@ -961,7 +982,7 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
 
                     // Look for any timeline event that's a reply to the redacted event, and redact
                     // the replied-to event there as well.
-                    Self::maybe_update_responses(self.items, &redacted, &new_item);
+                    Self::maybe_update_responses(self.meta, self.items, &redacted, &new_item);
 
                     self.items.replace(idx, TimelineItem::new(new_item, internal_id));
                     self.result.items_updated += 1;
@@ -974,43 +995,42 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
         };
     }
 
-    /// Attempts to redact a reaction, local or remote.
+    /// Attempts to redact an aggregation (e.g. a reaction, a poll response,
+    /// etc.).
     ///
     /// Returns true if it's succeeded.
-    #[instrument(skip_all, fields(redacts = ?reaction_id))]
-    fn handle_reaction_redaction(&mut self, reaction_id: TimelineEventItemId) -> bool {
-        if let Some(FullReactionKey {
-            item: TimelineEventItemId::EventId(reacted_to_event_id),
-            key,
-            sender,
-        }) = self.meta.reactions.map.remove(&reaction_id)
-        {
-            let Some((item_pos, item)) = rfind_event_by_id(self.items, &reacted_to_event_id) else {
-                // The remote event wasn't in the timeline.
-                if let TimelineEventItemId::EventId(event_id) = reaction_id {
-                    // Remove any possibly pending reactions to that event, as this redaction would
-                    // affect them.
-                    if let Some(reactions) =
-                        self.meta.reactions.pending.get_mut(&reacted_to_event_id)
-                    {
-                        reactions.swap_remove(&event_id);
-                    }
+    #[instrument(skip_all, fields(redacts = ?aggregation_id))]
+    fn handle_aggregation_redaction(&mut self, aggregation_id: OwnedEventId) -> bool {
+        let aggregation_id = TimelineEventItemId::EventId(aggregation_id);
+
+        let Some((target, aggregation)) =
+            self.meta.aggregations.try_remove_aggregation(&aggregation_id)
+        else {
+            // This wasn't a known aggregation that was redacted.
+            return false;
+        };
+
+        if let Some((item_pos, item)) = rfind_event_by_item_id(self.items, target) {
+            let mut content = item.content().clone();
+            match aggregation.unapply(&mut content) {
+                ApplyAggregationResult::UpdatedItem => {
+                    trace!("removed aggregation");
+                    let internal_id = item.internal_id.to_owned();
+                    let new_item = item.with_content(content);
+                    self.items.replace(item_pos, TimelineItem::new(new_item, internal_id));
+                    self.result.items_updated += 1;
+                }
+                ApplyAggregationResult::LeftItemIntact => {}
+                ApplyAggregationResult::Error(err) => {
+                    warn!("error when unapplying aggregation: {err}");
                 }
-
-                // We haven't redacted the reaction.
-                return false;
-            };
-
-            let mut reactions = item.reactions.clone();
-            if reactions.remove_reaction(&sender, &key).is_some() {
-                trace!("Removing reaction");
-                self.items.replace(item_pos, item.with_reactions(reactions));
-                self.result.items_updated += 1;
-                return true;
             }
+        } else {
+            info!("missing related-to item ({target:?}) for aggregation {aggregation_id:?}");
         }
 
-        false
+        // In all cases, we noticed this was an aggregation.
+        true
     }
 
     /// Add a new event item in the timeline.
@@ -1018,7 +1038,8 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
     /// # Safety
     ///
     /// This method is not marked as unsafe **but** it manipulates
-    /// [`TimelineMetadata::all_remote_events`]. 2 rules **must** be respected:
+    /// [`ObservableItemsTransaction::all_remote_events`]. 2 rules **must** be
+    /// respected:
     ///
     /// 1. the remote event of the item being added **must** be present in
     ///    `all_remote_events`,
@@ -1026,15 +1047,21 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
     ///    timeline item being added here.
     fn add_item(
         &mut self,
-        content: TimelineItemContent,
+        mut content: TimelineItemContent,
         edit_json: Option<Raw<AnySyncTimelineEvent>>,
     ) {
         self.result.item_added = true;
 
+        // Apply any pending or stashed aggregations.
+        if let Err(err) =
+            self.meta.aggregations.apply(&self.ctx.flow.timeline_item_id(), &mut content)
+        {
+            warn!("discarding aggregations: {err}");
+        }
+
         let sender = self.ctx.sender.to_owned();
         let sender_profile = TimelineDetails::from_initial_value(self.ctx.sender_profile.clone());
         let timestamp = self.ctx.timestamp;
-        let reactions = self.pending_reactions(&content).unwrap_or_default();
 
         let kind: EventTimelineItemKind = match &self.ctx.flow {
             Flow::Local { txn_id, send_handle } => LocalEventTimelineItem {
@@ -1055,7 +1082,7 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
                         .as_event()
                         .and_then(|ev| Some(ev.as_remote()?.origin))
                         .unwrap_or_else(|| {
-                            error!("Decryption retried on a local event");
+                            error!("Tried to update a local event");
                             RemoteEventOrigin::Unknown
                         }),
                 };
@@ -1075,15 +1102,14 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
             }
         };
 
-        let is_room_encrypted = self.meta.is_room_encrypted.read().unwrap_or_default();
+        let is_room_encrypted = self.meta.is_room_encrypted;
 
-        let mut item = EventTimelineItem::new(
+        let item = EventTimelineItem::new(
             sender,
             sender_profile,
             timestamp,
             content,
             kind,
-            reactions,
             is_room_encrypted,
         );
 
@@ -1099,13 +1125,13 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
             Flow::Remote {
                 position: TimelineItemPosition::Start { .. }, event_id, txn_id, ..
             } => {
-                let removed_duplicated_timeline_item = Self::deduplicate_local_timeline_item(
+                let item = Self::recycle_local_or_create_item(
                     self.items,
-                    &mut item,
-                    Some(event_id),
-                    txn_id.as_ref().map(AsRef::as_ref),
+                    self.meta,
+                    item,
+                    event_id,
+                    txn_id.as_deref(),
                 );
-                let item = new_timeline_item(self.meta, item, removed_duplicated_timeline_item);
 
                 trace!("Adding new remote timeline item at the start");
 
@@ -1118,13 +1144,13 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
                 txn_id,
                 ..
             } => {
-                let removed_duplicated_timeline_item = Self::deduplicate_local_timeline_item(
+                let item = Self::recycle_local_or_create_item(
                     self.items,
-                    &mut item,
-                    Some(event_id),
-                    txn_id.as_ref().map(AsRef::as_ref),
+                    self.meta,
+                    item,
+                    event_id,
+                    txn_id.as_deref(),
                 );
-                let item = new_timeline_item(self.meta, item, removed_duplicated_timeline_item);
 
                 let all_remote_events = self.items.all_remote_events();
                 let event_index = *event_index;
@@ -1156,7 +1182,17 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
                             (!timeline_item.as_event()?.is_local_echo())
                                 .then_some(timeline_item_index + 1)
                         })
-                        .unwrap_or(0)
+                        .unwrap_or_else(|| {
+                            // We don't have any local echo, so we could insert at 0. However, in
+                            // the case of an insertion caused by a pagination, we
+                            // may have already pushed the start of the timeline item, so we need
+                            // to check if the first item is that, and insert after it otherwise.
+                            if self.items.get(0).is_some_and(|item| item.is_timeline_start()) {
+                                1
+                            } else {
+                                0
+                            }
+                        })
                 });
 
                 trace!(
@@ -1171,13 +1207,13 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
             Flow::Remote {
                 position: TimelineItemPosition::End { .. }, event_id, txn_id, ..
             } => {
-                let removed_duplicated_timeline_item = Self::deduplicate_local_timeline_item(
+                let item = Self::recycle_local_or_create_item(
                     self.items,
-                    &mut item,
-                    Some(event_id),
-                    txn_id.as_ref().map(AsRef::as_ref),
+                    self.meta,
+                    item,
+                    event_id,
+                    txn_id.as_deref(),
                 );
-                let item = new_timeline_item(self.meta, item, removed_duplicated_timeline_item);
 
                 // Local events are always at the bottom. Let's find the latest remote event
                 // and insert after it, otherwise, if there is no remote event, insert at 0.
@@ -1236,7 +1272,7 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
                 trace!("Updating timeline item at position {idx}");
 
                 // Update all events that replied to this previously encrypted message.
-                Self::maybe_update_responses(self.items, decrypted_event_id, &item);
+                Self::maybe_update_responses(self.meta, self.items, decrypted_event_id, &item);
 
                 let internal_id = self.items[*idx].internal_id.clone();
                 self.items.replace(*idx, TimelineItem::new(item, internal_id));
@@ -1249,16 +1285,18 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
         }
     }
 
-    /// Remove the local timeline item matching the `event_id` or the
-    /// `transaction_id` of `new_event_timeline_item` if it exists.
-    // Note: this method doesn't take `&mut self` to avoid a borrow checker
-    // conflict with `TimelineEventHandler::add_item`.
-    fn deduplicate_local_timeline_item(
+    /// Try to recycle a local timeline item for the same event, or create a new
+    /// timeline item for it.
+    ///
+    /// Note: this method doesn't take `&mut self` to avoid a borrow checker
+    /// conflict with `TimelineEventHandler::add_item`.
+    fn recycle_local_or_create_item(
         items: &mut ObservableItemsTransaction<'_>,
-        new_event_timeline_item: &mut EventTimelineItem,
-        event_id: Option<&EventId>,
+        meta: &mut TimelineMetadata,
+        mut new_item: EventTimelineItem,
+        event_id: &EventId,
         transaction_id: Option<&TransactionId>,
-    ) -> Option<Arc<TimelineItem>> {
+    ) -> Arc<TimelineItem> {
         // Detect a local timeline item that matches `event_id` or `transaction_id`.
         if let Some((local_timeline_item_index, local_timeline_item)) = items
             .iter()
@@ -1287,7 +1325,7 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
                     return ControlFlow::Break(None);
                 }
 
-                if event_id == event_timeline_item.event_id()
+                if Some(event_id) == event_timeline_item.event_id()
                     || (transaction_id.is_some()
                         && transaction_id == event_timeline_item.transaction_id())
                 {
@@ -1309,71 +1347,66 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
                 "Removing local timeline item"
             );
 
-            transfer_details(new_event_timeline_item, local_timeline_item);
+            transfer_details(&mut new_item, local_timeline_item);
 
             // Remove the local timeline item.
-            return Some(items.remove(local_timeline_item_index));
-        };
-
-        None
+            let recycled = items.remove(local_timeline_item_index);
+            TimelineItem::new(new_item, recycled.internal_id.clone())
+        } else {
+            // We haven't found a matching local item to recycle; create a new item.
+            meta.new_timeline_item(new_item)
+        }
     }
 
     /// After updating the timeline item `new_item` which id is
     /// `target_event_id`, update other items that are responses to this item.
     fn maybe_update_responses(
+        meta: &mut TimelineMetadata,
         items: &mut ObservableItemsTransaction<'_>,
         target_event_id: &EventId,
         new_item: &EventTimelineItem,
     ) {
-        items.for_each(|mut entry| {
-            let Some(event_item) = entry.as_event() else { return };
-            let Some(message) = event_item.content.as_message() else { return };
-            let Some(in_reply_to) = message.in_reply_to() else { return };
-            if target_event_id == in_reply_to.event_id {
-                trace!(reply_event_id = ?event_item.identifier(), "Updating response to edited event");
-                let in_reply_to = InReplyToDetails {
-                    event_id: in_reply_to.event_id.clone(),
-                    event: TimelineDetails::Ready(Box::new(
-                        RepliedToEvent::from_timeline_item(new_item),
-                    )),
-                };
-                let new_reply_content =
-                TimelineItemContent::Message(message.with_in_reply_to(in_reply_to));
-                let new_reply_item =
-                entry.with_kind(event_item.with_content(new_reply_content));
-                ObservableItemsTransactionEntry::replace(&mut entry, new_reply_item);
-            }
-        });
-    }
+        let Some(replies) = meta.replies.get(target_event_id) else {
+            trace!("item has no replies");
+            return;
+        };
 
-    fn pending_reactions(
-        &mut self,
-        content: &TimelineItemContent,
-    ) -> Option<ReactionsByKeyBySender> {
-        // Drop pending reactions if the message is redacted.
-        if let TimelineItemContent::RedactedMessage = content {
-            return None;
+        for reply_id in replies {
+            let Some(timeline_item_index) = items
+                .get_remote_event_by_event_id(reply_id)
+                .and_then(|meta| meta.timeline_item_index)
+            else {
+                warn!(%reply_id, "event not known as an item in the timeline");
+                continue;
+            };
+
+            let Some(item) = items.get(timeline_item_index) else {
+                warn!(%reply_id, timeline_item_index, "mapping from event id to timeline item likely incorrect");
+                continue;
+            };
+
+            let Some(event_item) = item.as_event() else { continue };
+            let Some(msglike) = event_item.content.as_msglike() else { continue };
+            let Some(message) = msglike.as_message() else { continue };
+            let Some(in_reply_to) = msglike.in_reply_to.as_ref() else { continue };
+
+            trace!(reply_event_id = ?event_item.identifier(), "Updating response to updated event");
+            let in_reply_to = Some(InReplyToDetails {
+                event_id: in_reply_to.event_id.clone(),
+                event: TimelineDetails::Ready(Box::new(RepliedToEvent::from_timeline_item(
+                    new_item,
+                ))),
+            });
+
+            let new_reply_content = TimelineItemContent::MsgLike(MsgLikeContent {
+                kind: MsgLikeKind::Message(message.clone()),
+                reactions: msglike.reactions.clone(),
+                thread_root: msglike.thread_root.clone(),
+                in_reply_to,
+            });
+            let new_reply_item = item.with_kind(event_item.with_content(new_reply_content));
+            items.replace(timeline_item_index, new_reply_item);
         }
-
-        self.ctx.flow.event_id().and_then(|event_id| {
-            let reactions = self.meta.reactions.pending.remove(event_id)?;
-            let mut bundled = ReactionsByKeyBySender::default();
-
-            for (reaction_event_id, reaction) in reactions {
-                let group: &mut IndexMap<OwnedUserId, ReactionInfo> =
-                    bundled.entry(reaction.key).or_default();
-
-                group.insert(
-                    reaction.sender_id,
-                    ReactionInfo {
-                        timestamp: reaction.timestamp,
-                        status: ReactionStatus::RemoteToRemote(reaction_event_id),
-                    },
-                );
-            }
-
-            Some(bundled)
-        })
     }
 }
 
@@ -1385,33 +1418,17 @@ impl<'a, 'o> TimelineEventHandler<'a, 'o> {
 /// `old_item` *should* always be a local timeline item usually, but it
 /// can be a remote timeline item.
 fn transfer_details(new_item: &mut EventTimelineItem, old_item: &EventTimelineItem) {
-    let TimelineItemContent::Message(msg) = &mut new_item.content else { return };
-    let TimelineItemContent::Message(old_msg) = &old_item.content else { return };
+    let TimelineItemContent::MsgLike(new_msglike) = &mut new_item.content else {
+        return;
+    };
+    let TimelineItemContent::MsgLike(old_msglike) = &old_item.content else {
+        return;
+    };
 
-    let Some(in_reply_to) = &mut msg.in_reply_to else { return };
-    let Some(old_in_reply_to) = &old_msg.in_reply_to else { return };
+    let Some(in_reply_to) = &mut new_msglike.in_reply_to else { return };
+    let Some(old_in_reply_to) = &old_msglike.in_reply_to else { return };
 
     if matches!(&in_reply_to.event, TimelineDetails::Unavailable) {
         in_reply_to.event = old_in_reply_to.event.clone();
     }
 }
-
-/// Create a new timeline item from an [`EventTimelineItem`].
-///
-/// It is possible that the new timeline item replaces a duplicated timeline
-/// event (see [`TimelineEventHandler::deduplicate_local_timeline_item`]) in
-/// case it replaces a local timeline item.
-fn new_timeline_item(
-    metadata: &mut TimelineMetadata,
-    event_timeline_item: EventTimelineItem,
-    replaced_timeline_item: Option<Arc<TimelineItem>>,
-) -> Arc<TimelineItem> {
-    match replaced_timeline_item {
-        // Reuse the internal ID.
-        Some(to_replace_timeline_item) => {
-            TimelineItem::new(event_timeline_item, to_replace_timeline_item.internal_id.clone())
-        }
-
-        None => metadata.new_timeline_item(event_timeline_item),
-    }
-}
