@@ -12,41 +12,44 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::time::Duration;
+use std::ops::Not;
 
 use assert_matches::assert_matches;
 use assert_matches2::assert_let;
 use eyeball_im::VectorDiff;
 use futures_util::StreamExt;
 use matrix_sdk::{
-    config::SyncSettings,
-    test_utils::{logged_in_client_with_server, mocks::MatrixMockServer},
+    linked_chunk::{ChunkIdentifier, Position, Update},
+    test_utils::mocks::MatrixMockServer,
 };
 use matrix_sdk_test::{
-    async_test, event_factory::EventFactory, mocks::mock_encryption_state, sync_timeline_event,
-    JoinedRoomBuilder, RoomAccountDataTestEvent, StateTestEvent, SyncResponseBuilder, BOB,
+    async_test, event_factory::EventFactory, JoinedRoomBuilder, RoomAccountDataTestEvent,
+    StateTestEvent, ALICE, BOB,
 };
 use matrix_sdk_ui::{
     timeline::{
         AnyOtherFullStateEventContent, Error, EventSendState, RedactError, RoomExt,
         TimelineEventItemId, TimelineItemContent, VirtualTimelineItem,
     },
-    RoomListService, Timeline,
+    Timeline,
 };
 use ruma::{
     event_id,
-    events::room::{encryption::RoomEncryptionEventContent, message::RoomMessageEventContent},
-    owned_event_id, room_id, user_id, MilliSecondsSinceUnixEpoch,
+    events::room::{
+        encryption::RoomEncryptionEventContent,
+        message::{RedactedRoomMessageEventContent, RoomMessageEventContent},
+    },
+    owned_event_id, room_id, user_id, EventId, MilliSecondsSinceUnixEpoch,
 };
 use serde_json::json;
+use sliding_sync::assert_timeline_stream;
 use stream_assert::assert_pending;
 use wiremock::{
     matchers::{header, method, path_regex},
     Mock, ResponseTemplate,
 };
 
-use crate::mock_sync;
-
+mod decryption;
 mod echo;
 mod edit;
 mod focus_event;
@@ -64,53 +67,31 @@ pub(crate) mod sliding_sync;
 
 #[async_test]
 async fn test_reaction() {
+    let server = MatrixMockServer::new().await;
+    let client = server.client_builder().build().await;
+
     let room_id = room_id!("!a98sd12bjh:example.org");
-    let (client, server) = logged_in_client_with_server().await;
-    let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
+    let room = server.sync_joined_room(&client, room_id).await;
 
-    let mut sync_builder = SyncResponseBuilder::new();
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id));
+    server.mock_room_state_encryption().plain().mount().await;
 
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
-
-    mock_encryption_state(&server, false).await;
-
-    let room = client.get_room(room_id).unwrap();
     let timeline = room.timeline().await.unwrap();
     let (_, mut timeline_stream) = timeline.subscribe().await;
 
-    sync_builder.add_joined_room(
-        JoinedRoomBuilder::new(room_id)
-            .add_timeline_event(sync_timeline_event!({
-                "content": {
-                    "body": "hello",
-                    "msgtype": "m.text",
-                },
-                "event_id": "$TTvQUp1e17qkw41rBSjpZ",
-                "origin_server_ts": 152037280,
-                "sender": "@alice:example.org",
-                "type": "m.room.message",
-            }))
-            .add_timeline_event(sync_timeline_event!({
-                "content": {
-                    "m.relates_to": {
-                        "event_id": "$TTvQUp1e17qkw41rBSjpZ",
-                        "key": "👍",
-                        "rel_type": "m.annotation",
-                    },
-                },
-                "event_id": "$031IXQRi27504",
-                "origin_server_ts": 152038300,
-                "sender": "@bob:example.org",
-                "type": "m.reaction",
-            })),
-    );
+    let msg_event_id = event_id!("$msg");
+    let reaction_event_id = event_id!("$reaction");
 
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    let f = EventFactory::new();
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id)
+                .add_timeline_event(f.text_msg("hello").sender(&ALICE).event_id(msg_event_id))
+                .add_timeline_event(
+                    f.reaction(msg_event_id, "👍").sender(&BOB).event_id(reaction_event_id),
+                ),
+        )
+        .await;
 
     assert_let!(Some(timeline_updates) = timeline_stream.next().await);
     assert_eq!(timeline_updates.len(), 4);
@@ -118,118 +99,87 @@ async fn test_reaction() {
     // The new message starts with their author's read receipt.
     assert_let!(VectorDiff::PushBack { value: message } = &timeline_updates[0]);
     let event_item = message.as_event().unwrap();
-    assert_matches!(event_item.content(), TimelineItemContent::Message(_));
+    assert!(event_item.content().is_message());
     assert_eq!(event_item.read_receipts().len(), 1);
 
     // The new message is getting the reaction, which implies an implicit read
     // receipt that's obtained first.
     assert_let!(VectorDiff::Set { index: 0, value: updated_message } = &timeline_updates[1]);
     let event_item = updated_message.as_event().unwrap();
-    assert_let!(TimelineItemContent::Message(msg) = event_item.content());
+    assert_let!(Some(msg) = event_item.content().as_message());
     assert!(!msg.is_edited());
     assert_eq!(event_item.read_receipts().len(), 2);
-    assert_eq!(event_item.reactions().len(), 0);
+    assert_eq!(event_item.content().reactions().len(), 0);
 
     // Then the reaction is taken into account.
     assert_let!(VectorDiff::Set { index: 0, value: updated_message } = &timeline_updates[2]);
     let event_item = updated_message.as_event().unwrap();
-    assert_let!(TimelineItemContent::Message(msg) = event_item.content());
+    assert_let!(Some(msg) = event_item.content().as_message());
     assert!(!msg.is_edited());
     assert_eq!(event_item.read_receipts().len(), 2);
-    assert_eq!(event_item.reactions().len(), 1);
-    let group = &event_item.reactions()["👍"];
+    assert_eq!(event_item.content().reactions().len(), 1);
+    let group = &event_item.content().reactions()["👍"];
     assert_eq!(group.len(), 1);
     let senders: Vec<_> = group.keys().collect();
-    assert_eq!(senders.as_slice(), [user_id!("@bob:example.org")]);
+    assert_eq!(senders.as_slice(), [*BOB]);
 
     // The date divider.
     assert_let!(VectorDiff::PushFront { value: date_divider } = &timeline_updates[3]);
     assert!(date_divider.is_date_divider());
 
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id).add_timeline_event(
-        sync_timeline_event!({
-            "content": {},
-            "redacts": "$031IXQRi27504",
-            "event_id": "$N6eUCBc3vu58PL8TobGaVQzM",
-            "sender": "@bob:example.org",
-            "origin_server_ts": 152037280,
-            "type": "m.room.redaction",
-        }),
-    ));
+    assert_pending!(timeline_stream);
 
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id)
+                .add_timeline_event(f.redaction(reaction_event_id).sender(&BOB)),
+        )
+        .await;
 
     assert_let!(Some(timeline_updates) = timeline_stream.next().await);
     assert_eq!(timeline_updates.len(), 1);
 
     assert_let!(VectorDiff::Set { index: 1, value: updated_message } = &timeline_updates[0]);
     let event_item = updated_message.as_event().unwrap();
-    assert_let!(TimelineItemContent::Message(msg) = event_item.content());
+    assert_let!(Some(msg) = event_item.content().as_message());
     assert!(!msg.is_edited());
-    assert_eq!(event_item.reactions().len(), 0);
+    assert_eq!(event_item.content().reactions().len(), 0);
 
     assert_pending!(timeline_stream);
 }
 
 #[async_test]
 async fn test_redacted_message() {
+    let server = MatrixMockServer::new().await;
+    let client = server.client_builder().build().await;
+
     let room_id = room_id!("!a98sd12bjh:example.org");
-    let (client, server) = logged_in_client_with_server().await;
-    let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
+    let room = server.sync_joined_room(&client, room_id).await;
 
-    let mut sync_builder = SyncResponseBuilder::new();
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id));
+    server.mock_room_state_encryption().plain().mount().await;
 
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
-
-    mock_encryption_state(&server, false).await;
-
-    let room = client.get_room(room_id).unwrap();
     let timeline = room.timeline().await.unwrap();
     let (_, mut timeline_stream) = timeline.subscribe().await;
 
-    sync_builder.add_joined_room(
-        JoinedRoomBuilder::new(room_id)
-            .add_timeline_event(sync_timeline_event!({
-                "content": {},
-                "event_id": "$eeG0HA0FAZ37wP8kXlNkxx3I",
-                "origin_server_ts": 152035910,
-                "sender": "@alice:example.org",
-                "type": "m.room.message",
-                "unsigned": {
-                    "redacted_because": {
-                        "content": {},
-                        "redacts": "$eeG0HA0FAZ37wP8kXlNkxx3I",
-                        "event_id": "$N6eUCBc3vu58PL8TobGaVQzM",
-                        "sender": "@alice:example.org",
-                        "origin_server_ts": 152037280,
-                        "type": "m.room.redaction",
-                    },
-                },
-            }))
-            .add_timeline_event(sync_timeline_event!({
-                "content": {},
-                "redacts": "$eeG0HA0FAZ37wP8kXlNkxx3I",
-                "event_id": "$N6eUCBc3vu58PL8TobGaVQzM",
-                "sender": "@alice:example.org",
-                "origin_server_ts": 152037280,
-                "type": "m.room.redaction",
-            })),
-    );
-
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    let event_id = event_id!("$eeG0HA0FAZ37wP8kXlNkxx3I");
+    let f = EventFactory::new();
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id)
+                .add_timeline_event(
+                    f.redacted(*ALICE, RedactedRoomMessageEventContent::new()).event_id(event_id),
+                )
+                .add_timeline_event(f.redaction(event_id).sender(&ALICE)),
+        )
+        .await;
 
     assert_let!(Some(timeline_updates) = timeline_stream.next().await);
     assert_eq!(timeline_updates.len(), 2);
 
     assert_let!(VectorDiff::PushBack { value: first } = &timeline_updates[0]);
-    assert_matches!(first.as_event().unwrap().content(), TimelineItemContent::RedactedMessage);
+    assert!(first.as_event().unwrap().content().is_redacted());
 
     assert_let!(VectorDiff::PushFront { value: date_divider } = &timeline_updates[1]);
     assert!(date_divider.is_date_divider());
@@ -375,21 +325,14 @@ async fn test_redact_local_sent_message() {
 
 #[async_test]
 async fn test_redact_nonexisting_item() {
+    let server = MatrixMockServer::new().await;
+    let client = server.client_builder().build().await;
+
     let room_id = room_id!("!a98sd12bjh:example.org");
-    let (client, server) = logged_in_client_with_server().await;
+    let room = server.sync_joined_room(&client, room_id).await;
 
-    let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
+    server.mock_room_state_encryption().plain().mount().await;
 
-    let mut sync_builder = SyncResponseBuilder::new();
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id));
-
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
-
-    mock_encryption_state(&server, false).await;
-
-    let room = client.get_room(room_id).unwrap();
     let timeline = room.timeline().await.unwrap();
 
     let error = timeline
@@ -405,81 +348,62 @@ async fn test_redact_nonexisting_item() {
 
 #[async_test]
 async fn test_read_marker() {
+    let server = MatrixMockServer::new().await;
+    let client = server.client_builder().build().await;
+
     let room_id = room_id!("!a98sd12bjh:example.org");
-    let (client, server) = logged_in_client_with_server().await;
-    let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
+    let room = server.sync_joined_room(&client, room_id).await;
 
-    let mut sync_builder = SyncResponseBuilder::new();
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id));
+    server.mock_room_state_encryption().plain().mount().await;
 
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
-
-    mock_encryption_state(&server, false).await;
-
-    let room = client.get_room(room_id).unwrap();
     let timeline = room.timeline().await.unwrap();
     let (_, mut timeline_stream) = timeline.subscribe().await;
 
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id).add_timeline_event(
-        sync_timeline_event!({
-            "content": {
-                "body": "hello",
-                "msgtype": "m.text",
-            },
-            "event_id": "$someplace:example.org",
-            "origin_server_ts": 152037280,
-            "sender": "@alice:example.org",
-            "type": "m.room.message",
-        }),
-    ));
-
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    let f = EventFactory::new();
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id).add_timeline_event(
+                f.text_msg("hello").sender(&ALICE).event_id(event_id!("$someplace:example.org")),
+            ),
+        )
+        .await;
 
     assert_let!(Some(timeline_updates) = timeline_stream.next().await);
     assert_eq!(timeline_updates.len(), 2);
 
     assert_let!(VectorDiff::PushBack { value: message } = &timeline_updates[0]);
-    assert_matches!(message.as_event().unwrap().content(), TimelineItemContent::Message(_));
+    assert!(message.as_event().unwrap().content().is_message());
 
     assert_let!(VectorDiff::PushFront { value: date_divider } = &timeline_updates[1]);
     assert!(date_divider.is_date_divider());
 
-    sync_builder.add_joined_room(
-        JoinedRoomBuilder::new(room_id).add_account_data(RoomAccountDataTestEvent::FullyRead),
-    );
-
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id).add_account_data(RoomAccountDataTestEvent::FullyRead),
+        )
+        .await;
 
     // Nothing should happen, the marker cannot be added at the end.
+    assert_pending!(timeline_stream);
 
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id).add_timeline_event(
-        sync_timeline_event!({
-            "content": {
-                "body": "hello to you!",
-                "msgtype": "m.text",
-            },
-            "event_id": "$someotherplace:example.org",
-            "origin_server_ts": 152067280,
-            "sender": "@bob:example.org",
-            "type": "m.room.message",
-        }),
-    ));
-
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id).add_timeline_event(
+                f.text_msg("hello to you!")
+                    .sender(&BOB)
+                    .event_id(event_id!("$someotherplace:example.org")),
+            ),
+        )
+        .await;
 
     assert_let!(Some(timeline_updates) = timeline_stream.next().await);
     assert_eq!(timeline_updates.len(), 2);
 
     assert_let!(VectorDiff::PushBack { value: message } = &timeline_updates[0]);
-    assert_matches!(message.as_event().unwrap().content(), TimelineItemContent::Message(_));
+    assert!(message.as_event().unwrap().content().is_message());
 
     assert_let!(VectorDiff::Insert { index: 2, value: marker } = &timeline_updates[1]);
     assert_matches!(marker.as_virtual().unwrap(), VirtualTimelineItem::ReadMarker);
@@ -489,45 +413,34 @@ async fn test_read_marker() {
 
 #[async_test]
 async fn test_sync_highlighted() {
-    let room_id = room_id!("!a98sd12bjh:example.org");
-    let (client, server) = logged_in_client_with_server().await;
-    let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
+    let server = MatrixMockServer::new().await;
+    let client = server.client_builder().build().await;
 
-    let mut sync_builder = SyncResponseBuilder::new();
-    sync_builder
-        // We need the member event and power levels locally so the push rules processor works.
-        .add_joined_room(
+    server.mock_room_state_encryption().plain().mount().await;
+
+    let room_id = room_id!("!a98sd12bjh:example.org");
+    let room = server
+        .sync_room(
+            &client,
+            // We need the member event and power levels locally so the push rules processor works.
             JoinedRoomBuilder::new(room_id)
                 .add_state_event(StateTestEvent::Member)
                 .add_state_event(StateTestEvent::PowerLevels),
-        );
+        )
+        .await;
 
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
-
-    mock_encryption_state(&server, false).await;
-
-    let room = client.get_room(room_id).unwrap();
     let timeline = room.timeline().await.unwrap();
     let (_, mut timeline_stream) = timeline.subscribe().await;
 
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id).add_timeline_event(
-        sync_timeline_event!({
-            "content": {
-                "body": "hello",
-                "msgtype": "m.text",
-            },
-            "event_id": "$msda7m0df9E9op3",
-            "origin_server_ts": 152037280,
-            "sender": "@example:localhost",
-            "type": "m.room.message",
-        }),
-    ));
-
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    let f = EventFactory::new();
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id).add_timeline_event(
+                f.text_msg("hello").sender(&ALICE).event_id(event_id!("$msda7m0df9E9op3")),
+            ),
+        )
+        .await;
 
     assert_let!(Some(timeline_updates) = timeline_stream.next().await);
     assert_eq!(timeline_updates.len(), 2);
@@ -540,23 +453,15 @@ async fn test_sync_highlighted() {
     assert_let!(VectorDiff::PushFront { value: date_divider } = &timeline_updates[1]);
     assert!(date_divider.is_date_divider());
 
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id).add_timeline_event(
-        sync_timeline_event!({
-            "content": {
-                "body": "This room has been replaced",
-                "replacement_room": "!newroom:localhost",
-            },
-            "event_id": "$foun39djjod0f",
-            "origin_server_ts": 152039280,
-            "sender": "@bob:localhost",
-            "state_key": "",
-            "type": "m.room.tombstone",
-        }),
-    ));
-
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id).add_timeline_event(
+                f.room_tombstone("This room has been replaced", room_id!("!newroom:localhost"))
+                    .sender(&BOB),
+            ),
+        )
+        .await;
 
     assert_let!(Some(timeline_updates) = timeline_stream.next().await);
     assert_eq!(timeline_updates.len(), 1);
@@ -571,20 +476,14 @@ async fn test_sync_highlighted() {
 
 #[async_test]
 async fn test_duplicate_maintains_correct_order() {
+    let server = MatrixMockServer::new().await;
+    let client = server.client_builder().build().await;
+
     let room_id = room_id!("!a98sd12bjh:example.org");
-    let (client, server) = logged_in_client_with_server().await;
-    let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
+    let room = server.sync_joined_room(&client, room_id).await;
 
-    let mut sync_builder = SyncResponseBuilder::new();
-    sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id));
+    server.mock_room_state_encryption().plain().mount().await;
 
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
-
-    mock_encryption_state(&server, false).await;
-
-    let room = client.get_room(room_id).unwrap();
     let timeline = room.timeline().await.unwrap();
 
     // At the beginning, the timeline is empty.
@@ -593,14 +492,13 @@ async fn test_duplicate_maintains_correct_order() {
     let f = EventFactory::new().sender(user_id!("@a:b.c"));
 
     // We receive an event F, from a sliding sync with timeline limit=1.
-    sync_builder.add_joined_room(
-        JoinedRoomBuilder::new(room_id)
-            .add_timeline_event(f.text_msg("C").event_id(event_id!("$c")).into_raw_sync()),
-    );
-
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id)
+                .add_timeline_event(f.text_msg("C").event_id(event_id!("$c")).into_raw_sync()),
+        )
+        .await;
 
     // The timeline item represents the message we just received.
     let items = timeline.items().await;
@@ -612,16 +510,15 @@ async fn test_duplicate_maintains_correct_order() {
 
     // We receive multiple events, and C is now the last one (because we supposedly
     // increased the timeline limit).
-    sync_builder.add_joined_room(
-        JoinedRoomBuilder::new(room_id)
-            .add_timeline_event(f.text_msg("A").event_id(event_id!("$a")).into_raw_sync())
-            .add_timeline_event(f.text_msg("B").event_id(event_id!("$b")).into_raw_sync())
-            .add_timeline_event(f.text_msg("C").event_id(event_id!("$c")).into_raw_sync()),
-    );
-
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id)
+                .add_timeline_event(f.text_msg("A").event_id(event_id!("$a")).into_raw_sync())
+                .add_timeline_event(f.text_msg("B").event_id(event_id!("$b")).into_raw_sync())
+                .add_timeline_event(f.text_msg("C").event_id(event_id!("$c")).into_raw_sync()),
+        )
+        .await;
 
     let items = timeline.items().await;
     assert_eq!(items.len(), 4, "{items:?}");
@@ -644,16 +541,10 @@ async fn test_pin_event_is_sent_successfully() {
     assert!(!timeline.items().await.is_empty());
 
     // Pinning a remote event succeeds.
-    setup
-        .mock_pin_unpin_response(ResponseTemplate::new(200).set_body_json(json!({
-            "event_id": "$42"
-        })))
-        .await;
+    setup.server.mock_set_room_pinned_events().ok(owned_event_id!("$42")).mock_once().mount().await;
 
     let event_id = setup.event_id();
     assert!(timeline.pin_event(event_id).await.unwrap());
-
-    setup.reset_server().await;
 }
 
 #[async_test]
@@ -666,8 +557,6 @@ async fn test_pin_event_is_returning_false_because_is_already_pinned() {
 
     let event_id = setup.event_id();
     assert!(!timeline.pin_event(event_id).await.unwrap());
-
-    setup.reset_server().await;
 }
 
 #[async_test]
@@ -679,12 +568,10 @@ async fn test_pin_event_is_returning_an_error() {
     assert!(!timeline.items().await.is_empty());
 
     // Pinning a remote event fails.
-    setup.mock_pin_unpin_response(ResponseTemplate::new(400)).await;
+    setup.server.mock_set_room_pinned_events().unauthorized().mock_once().mount().await;
 
     let event_id = setup.event_id();
     assert!(timeline.pin_event(event_id).await.is_err());
-
-    setup.reset_server().await;
 }
 
 #[async_test]
@@ -696,16 +583,10 @@ async fn test_unpin_event_is_sent_successfully() {
     assert!(!timeline.items().await.is_empty());
 
     // Unpinning a remote event succeeds.
-    setup
-        .mock_pin_unpin_response(ResponseTemplate::new(200).set_body_json(json!({
-            "event_id": "$42"
-        })))
-        .await;
+    setup.server.mock_set_room_pinned_events().ok(owned_event_id!("$42")).mock_once().mount().await;
 
     let event_id = setup.event_id();
     assert!(timeline.unpin_event(event_id).await.unwrap());
-
-    setup.reset_server().await;
 }
 
 #[async_test]
@@ -718,8 +599,6 @@ async fn test_unpin_event_is_returning_false_because_is_not_pinned() {
 
     let event_id = setup.event_id();
     assert!(!timeline.unpin_event(event_id).await.unwrap());
-
-    setup.reset_server().await;
 }
 
 #[async_test]
@@ -731,67 +610,62 @@ async fn test_unpin_event_is_returning_an_error() {
     assert!(!timeline.items().await.is_empty());
 
     // Unpinning a remote event fails.
-    setup.mock_pin_unpin_response(ResponseTemplate::new(400)).await;
+    setup.server.mock_set_room_pinned_events().unauthorized().mock_once().mount().await;
 
     let event_id = setup.event_id();
     assert!(timeline.unpin_event(event_id).await.is_err());
-
-    setup.reset_server().await;
 }
 
 #[async_test]
 async fn test_timeline_without_encryption_info() {
-    // No encryption is mocked for this client/server pair
-    let (client, server) = logged_in_client_with_server().await;
-    let _ = RoomListService::new(client.clone()).await.unwrap();
+    // The room encryption state is NOT mocked on purpose.
+    let server = MatrixMockServer::new().await;
+    let client = server.client_builder().build().await;
+
+    client.event_cache().subscribe().unwrap();
 
     let room_id = room_id!("!a98sd12bjh:example.org");
-    let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
-    let f = EventFactory::new().room(room_id).sender(*BOB);
 
-    let mut sync_builder = SyncResponseBuilder::new();
-    sync_builder.add_joined_room(
-        JoinedRoomBuilder::new(room_id).add_timeline_event(f.text_msg("A message").into_raw_sync()),
-    );
+    let f = EventFactory::new();
+    let room = server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id)
+                .add_timeline_event(f.text_msg("A message").sender(*BOB)),
+        )
+        .await;
 
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
-
-    let room = client.get_room(room_id).unwrap();
     // Previously this would have panicked.
     let timeline = room.timeline().await.unwrap();
 
     let (items, _) = timeline.subscribe().await;
     assert_eq!(items.len(), 2);
     assert!(items[0].as_virtual().is_some());
-    // No encryption, no shields
+    // No encryption, no shields.
     assert!(items[1].as_event().unwrap().get_shield(false).is_none());
 }
 
 #[async_test]
 async fn test_timeline_without_encryption_can_update() {
-    // No encryption is mocked for this client/server pair
-    let (client, server) = logged_in_client_with_server().await;
-    let _ = RoomListService::new(client.clone()).await.unwrap();
+    // The room encryption state is NOT mocked on purpose.
+    let server = MatrixMockServer::new().await;
+    let client = server.client_builder().build().await;
+
+    client.event_cache().subscribe().unwrap();
 
     let room_id = room_id!("!jEsUZKDJdhlrceRyVU:example.org");
-    let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
+
     let f = EventFactory::new().room(room_id).sender(*BOB);
+    let room = server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id).add_timeline_event(f.text_msg("A message")),
+        )
+        .await;
 
-    let mut sync_builder = SyncResponseBuilder::new();
-    sync_builder.add_joined_room(
-        JoinedRoomBuilder::new(room_id).add_timeline_event(f.text_msg("A message").into_raw_sync()),
-    );
-
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
-
-    let room = client.get_room(room_id).unwrap();
     // Previously this would have panicked.
     // We're creating a timeline without read receipts tracking to check only the
-    // encryption changes
+    // encryption changes.
     let timeline = Timeline::builder(&room).build().await.unwrap();
 
     let (items, mut stream) = timeline.subscribe().await;
@@ -801,110 +675,163 @@ async fn test_timeline_without_encryption_can_update() {
     assert!(items[1].as_event().unwrap().get_shield(false).is_none());
 
     let encryption_event_content = RoomEncryptionEventContent::with_recommended_defaults();
-    sync_builder.add_joined_room(
-        JoinedRoomBuilder::new(room_id)
-            .add_timeline_event(f.event(encryption_event_content).state_key("").into_raw_sync())
-            .add_timeline_event(f.text_msg("An encrypted message").into_raw_sync()),
-    );
-    mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-    let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-    server.reset().await;
+    server
+        .sync_room(
+            &client,
+            JoinedRoomBuilder::new(room_id)
+                .add_timeline_event(f.event(encryption_event_content).state_key("").into_raw_sync())
+                .add_timeline_event(f.text_msg("An encrypted message").into_raw_sync()),
+        )
+        .await;
 
     assert_let!(Some(timeline_updates) = stream.next().await);
     assert_eq!(timeline_updates.len(), 3);
 
-    // Previous timeline event now has a shield
+    // Previous timeline event now has a shield.
     assert_let!(VectorDiff::Set { index, value } = &timeline_updates[0]);
     assert_eq!(*index, 1);
     assert!(value.as_event().unwrap().get_shield(false).is_some());
 
-    // Room encryption event is received
+    // Room encryption event is received.
     assert_let!(VectorDiff::PushBack { value } = &timeline_updates[1]);
     assert_let!(TimelineItemContent::OtherState(other_state) = value.as_event().unwrap().content());
     assert_let!(AnyOtherFullStateEventContent::RoomEncryption(_) = other_state.content());
     assert!(value.as_event().unwrap().get_shield(false).is_some());
 
-    // New message event is received and has a shield
+    // New message event is received and has a shield.
     assert_let!(VectorDiff::PushBack { value } = &timeline_updates[2]);
     assert!(value.as_event().unwrap().get_shield(false).is_some());
 
     assert_pending!(stream);
 }
 
+#[async_test]
+async fn test_timeline_receives_a_limited_number_of_events_when_subscribing() {
+    let room_id = room_id!("!foo:bar.baz");
+    let event_factory = EventFactory::new().room(room_id).sender(&ALICE);
+
+    let mock_server = MatrixMockServer::new().await;
+    let client = mock_server.client_builder().build().await;
+
+    mock_server.sync_joined_room(&client, room_id).await;
+
+    // Let's store events in the event cache _before_ the timeline is created.
+    {
+        let event_cache_store = client.event_cache_store().lock().await.unwrap();
+
+        // The event cache contains 30 events.
+        event_cache_store
+            .handle_linked_chunk_updates(
+                room_id,
+                vec![
+                    Update::NewItemsChunk {
+                        previous: None,
+                        new: ChunkIdentifier::new(42),
+                        next: None,
+                    },
+                    Update::PushItems {
+                        at: Position::new(ChunkIdentifier::new(42), 0),
+                        items: (0..30)
+                            .map(|nth| {
+                                event_factory
+                                    .text_msg("foo")
+                                    .event_id(&EventId::parse(format!("$ev{nth}")).unwrap())
+                                    .into_event()
+                            })
+                            .collect::<Vec<_>>(),
+                    },
+                ],
+            )
+            .await
+            .unwrap();
+    }
+
+    // Set up the event cache.
+    let event_cache = client.event_cache();
+    event_cache.subscribe().unwrap();
+    event_cache.enable_storage().unwrap();
+
+    let room = client.get_room(room_id).unwrap();
+
+    // Now the timeline is created.
+    let timeline = room.timeline().await.unwrap();
+
+    let (timeline_initial_items, mut timeline_stream) = timeline.subscribe().await;
+
+    // The timeline receives 20 initial values, not 30!
+    assert_eq!(timeline_initial_items.len(), 20);
+    assert_pending!(timeline_stream);
+
+    // To get the other, the timeline needs to paginate.
+    //
+    // Now let's do a backwards pagination of 5 items.
+    let hit_end_of_timeline = timeline.paginate_backwards(5).await.unwrap();
+
+    assert!(hit_end_of_timeline.not());
+
+    // Oh, 5 new items, without even hitting the network because the timeline
+    // already has these!
+    assert_timeline_stream! {
+        [timeline_stream]
+        prepend "$ev9";
+        prepend "$ev8";
+        prepend "$ev7";
+        prepend "$ev6";
+        prepend "$ev5";
+    };
+    assert_pending!(timeline_stream);
+}
+
 struct PinningTestSetup<'a> {
-    event_id: &'a ruma::EventId,
+    event_id: &'a EventId,
     room_id: &'a ruma::RoomId,
     client: matrix_sdk::Client,
-    server: wiremock::MockServer,
-    sync_settings: SyncSettings,
-    sync_builder: SyncResponseBuilder,
+    server: MatrixMockServer,
 }
 
 impl PinningTestSetup<'_> {
     async fn new() -> Self {
+        let server = MatrixMockServer::new().await;
+        let client = server.client_builder().build().await;
+
         let room_id = room_id!("!a98sd12bjh:example.org");
-        let (client, server) = logged_in_client_with_server().await;
-        let sync_settings = SyncSettings::new().timeout(Duration::from_millis(3000));
+        server.sync_joined_room(&client, room_id).await;
 
-        let mut sync_builder = SyncResponseBuilder::new();
-        let event_id = event_id!("$a");
-        sync_builder.add_joined_room(JoinedRoomBuilder::new(room_id));
-
-        mock_sync(&server, sync_builder.build_json_sync_response(), None).await;
-        let _response = client.sync_once(sync_settings.clone()).await.unwrap();
-        server.reset().await;
-
-        let setup = Self { event_id, room_id, client, server, sync_settings, sync_builder };
+        server.mock_room_state_encryption().plain().mount().await;
 
         // This is necessary to get an empty list of pinned events when there are no
         // pinned events state event in the required state
-        setup.mock_get_empty_pinned_events_state_response().await;
-
-        setup
-    }
-
-    async fn timeline(&self) -> Timeline {
-        mock_encryption_state(&self.server, false).await;
-        let room = self.client.get_room(self.room_id).unwrap();
-        room.timeline().await.unwrap()
-    }
-
-    async fn reset_server(&self) {
-        self.server.reset().await;
-    }
-
-    async fn mock_pin_unpin_response(&self, response: ResponseTemplate) {
-        Mock::given(method("PUT"))
-            .and(path_regex(r"^/_matrix/client/r0/rooms/.*/state/m.room.pinned_events/.*?"))
-            .and(header("authorization", "Bearer 1234"))
-            .respond_with(response)
-            .mount(&self.server)
-            .await;
-    }
-
-    async fn mock_get_empty_pinned_events_state_response(&self) {
         Mock::given(method("GET"))
             .and(path_regex(r"^/_matrix/client/r0/rooms/.*/state/m.room.pinned_events/.*"))
             .and(header("authorization", "Bearer 1234"))
             .respond_with(ResponseTemplate::new(404).set_body_json(json!({})))
-            .mount(&self.server)
+            .mount(server.server())
             .await;
+
+        let event_id = event_id!("$a");
+        Self { event_id, room_id, client, server }
+    }
+
+    async fn timeline(&self) -> Timeline {
+        let room = self.client.get_room(self.room_id).unwrap();
+        room.timeline().await.unwrap()
     }
 
     async fn mock_sync(&mut self, is_using_pinned_state_event: bool) {
         let f = EventFactory::new().sender(user_id!("@a:b.c"));
+
         let mut joined_room_builder = JoinedRoomBuilder::new(self.room_id)
             .add_timeline_event(f.text_msg("A").event_id(self.event_id).into_raw_sync());
+
         if is_using_pinned_state_event {
             joined_room_builder =
                 joined_room_builder.add_state_event(StateTestEvent::RoomPinnedEvents);
         }
-        self.sync_builder.add_joined_room(joined_room_builder);
-        mock_sync(&self.server, self.sync_builder.build_json_sync_response(), None).await;
-        let _response = self.client.sync_once(self.sync_settings.clone()).await.unwrap();
+
+        self.server.sync_room(&self.client, joined_room_builder).await;
     }
 
-    fn event_id(&self) -> &ruma::EventId {
+    fn event_id(&self) -> &EventId {
         self.event_id
     }
 }
